--------------------------------------------------------------------------------
--
--   FileName:         bit_serial_adder.vhd
--   Dependencies:     dff.vhd, full_adder.vhd
--
--   HDL CODE IS PROVIDED "AS IS."  DIGI-KEY EXPRESSLY DISCLAIMS ANY
--   WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
--   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
--   PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
--   BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
--   DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
--   PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
--   BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
--   ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
--
--   Version History
--   Version 1.0 12/14/16 -TR
--
--------------------------------------------------------------------------------

--Bit serial adder

library IEEE;
use IEEE.std_logic_1164.all;

entity bit_serial_adder is
  port (
    clk   : in std_logic;
    rst_n : in std_logic;
    a     : in std_logic;
    b     : in std_logic;
    s     : out std_logic
  );
end entity;

architecture arch of bit_serial_adder is
  --Internal signals and components
  signal carry_out, reg_out : std_logic;
  signal sum : std_logic;

  component dfflop is
    port (
  		clk	  :	in std_logic;
  		rst_n	:	in std_logic;
  		D     :	in std_logic;
  		Q		  :	out std_logic
  	);
  end component;

  component full_adder is
    port (
      A     : in std_logic;
      B     : in std_logic;
      Cin   : in std_logic;
      S     : out std_logic;
      Cout  : out std_logic
    );
  end component;

begin

  adder : full_adder
  port map (a, b, reg_out, sum, carry_out);

  carry_flip_flop : dfflop
  port map (clk, rst_n, carry_out, reg_out);

  output_flip_flop : dfflop
  port map (clk, rst_n, sum, s);

end architecture;
