--------------------------------------------------------------------------------
--
--   FileName:         shift_register.vhd
--   Dependencies:     None
--
--   HDL CODE IS PROVIDED "AS IS."  DIGI-KEY EXPRESSLY DISCLAIMS ANY
--   WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
--   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
--   PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
--   BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
--   DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
--   PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
--   BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
--   ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
--
--   Version History
--   Version 1.0 12/14/16 -TR
--
--------------------------------------------------------------------------------


--Serial/parallel shift register

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity shift_register is
  generic (data_width : natural := 8);
  port (
    rst_n       : in std_logic;
    clk         : in std_logic;
    s_p         : in std_logic;   --Selects between serial and parallel operation, serial input when high, parallel input when low
    dir         : in std_logic;   --Selects either a left or right shift, high for right, low for left
    reg_en_n    : in std_logic;   --Active low register enable, value in the register is held while this signal is high
    shift_in    : in std_logic;
    D           : in std_logic_vector(data_width-1 downto 0); --Parallel input
    Q           : out std_logic_vector(data_width-1 downto 0);
    serial_out  : out std_logic
  );
end entity;

architecture arch of shift_register is
  --Internal signals and components
  signal internal_reg : std_logic_vector(data_width-1 downto 0);

begin

  Q <= internal_reg;

  load : process(clk, rst_n, s_p, reg_en_n)
  begin
    if rst_n = '0' then
      internal_reg <= (others => '0');
    elsif rising_edge(clk) and s_p = '0' and reg_en_n = '0' then
      --Load in data on parallel input
      internal_reg <= D;
    elsif rising_edge(clk) and s_p = '1' and reg_en_n = '0' then
      if dir = '1' then
        --Shift right
        serial_out <= internal_reg(0);
        internal_reg(data_width-2 downto 0) <= internal_reg(data_width-1 downto 1);
        internal_reg(data_width-1) <= shift_in;
      else
        --Shift left
        serial_out <= internal_reg(data_width-1);
        internal_reg(data_width-1 downto 1) <= internal_reg(data_width-2 downto 0);
        internal_reg(0) <= shift_in;
      end if;
    end if;
  end process;


end architecture;
